#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <lib9.h>
#include <fmt.h>

#include "libaiff.h"

static int ofd = -1;
static int vfd = -1;
static int cfd = -1;
static int verbose;

enum {
	Bufsiz = 2048,
};

int
play(AIFF_Ref r, char *file)
{
	int dec[Bufsiz];
	uchar out[Bufsiz];
	int got, s, i, o;
	uvlong samps;				// # temporal sampling points in file
	int chans;					// # channels of audio
	double rate;				// samples per sec
	int bps;					// resolution, not important really. defines noise floor
	int segsz;					// do not care if I am using api below.


	if(AIFF_GetAudioFormat(r, &samps, &chans, &rate, &bps, &segsz) <= 0)
		sysfatal("%s cannot get audio format", file);

	if(verbose){
		print("enc aiff\n");
		print("chan %d\n", chans);
		print("freq %f\n", rate / 1000.0);
		print("rate %f\n", (rate * chans * bps) / 1024.0);
	}

	if(vfd < 0)
		fprint(2, "warning: audio sample rate is %d Hz\n", (int)rate);
	else{
		fprint(vfd, "speed %d\n", (int)rate);
		fprint(cfd, "rate %d\n", (int)rate);
	}
	o = 0;
	for(s = 0; s < samps; s += got){
		got = AIFF_ReadSamples32Bit(r, dec, Bufsiz);
		if(got == 0)
			sysfatal("%s unexpected EOF", file);

		for(i = 0; i < got; i += chans){
			out[o++] = dec[i] >> 16;
			out[o++] = dec[i] >> 24;

			if(chans == 1){
				out[o++] = dec[i] >> 16;
				out[o++] = dec[i] >> 24;
			}
			else{
				out[o++] = dec[i+1] >> 16;
				out[o++] = dec[i+1] >> 24;
			}

			if(o >= Bufsiz -4){
				if(write(ofd, out, o) != o)
					sysfatal("write failed, %r");
				o = 0;
			}
		}
	}
	if(write(ofd, out, o) != o)
		sysfatal("write failed, %r");
	return 0;
}


void
usage(void)
{
	fprint(2, "%s [-o outfile] file...\n", argv0);
	exit(1);
}

int
main(int argc, char *argv[])
{
	int i;
    AIFF_Ref r;
	char *outfile;

	outfile = NULL;
	ARGBEGIN{
	case 'v':
		verbose++;
		break;
	case 'o':
		outfile = EARGF(usage());
		break;
	default:
		usage();
	}ARGEND;

	if(outfile){
		if((ofd = creat(outfile, 0644)) < 0)
			sysfatal("%s cannot create, %r", outfile);
	}else{
		if((ofd = open("/dev/audio", O_WRONLY)) < 0)
			sysfatal("/dev/audio cannot open, %r");
		vfd = open("/dev/volume", O_WRONLY);
		cfd = open("/dev/audioctl", O_WRONLY);
	}

	if(argc == 0){	/* cannot easily read aiff from stdin with this library */
		usage();
	}
	else{
		for(i=0; i<argc; i++){
			if((r = AIFF_OpenFile(argv[i], F_RDONLY)) == nil)
				sysfatal("%s cannot open, %r", argv[i]);
			play(r, argv[i]);
			AIFF_CloseFile(r);
		}
	}
	return 0;
}
