#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <lib9.h>
#include <fmt.h>

#include "mp4ffv2.h"
#include "neaacdec.h"
#include "alac_decoder.h"

enum {
	Bufsiz = 8192
};

static int ofd = -1;
static int vfd = -1;
static int cfd = -1;

static uint
doread(void *v, void *buf, uint len)
{
	return fread(buf, 1, len, v);
}

static uint
doseek(void *v, uvlong off)
{
    return fseek(v, off, SEEK_SET);
}

static int
mp4aac(mp4ff_t *mp4, char *file, int track)
{
	ulong rate;
	ushort *dec;
	uint len, nmagic;
	NeAACDecHandle aac;
	NeAACDecFrameInfo inf;
	mp4ff_callback_t *mp4cb;
	mp4AudioSpecificConfig asc;
	NeAACDecConfigurationPtr cfg;
	int i, o, rc, nsamps, samp, got, inpalloc;
	uchar out[Bufsiz], *inp, chans, *magic;

    aac = NeAACDecOpen();
	cfg = NeAACDecGetCurrentConfiguration(aac);
    cfg->defSampleRate = 44100;
    cfg->defObjectType = LC;
    cfg->outputFormat = FAAD_FMT_16BIT;
    cfg->downMatrix = 0;
    cfg->useOldADTSFormat = 0;
	cfg->dontUpSampleImplicitSBR = 0;
	NeAACDecSetConfiguration(aac, cfg);

	if(mp4ff_get_decoder_config(mp4, track, &magic, &nmagic) != 0)
		sysfatal("%s track=%d cannot get config", file, track);

	if(NeAACDecInit2(aac, magic, nmagic, &rate, &chans) < 0)
		sysfatal("cannot initialise AAC decoder");


	o = 0;
	dec = nil;
	inp = NULL;
	inpalloc = 0;
	nsamps = mp4ff_num_samples(mp4, track);

	for(samp = 0; samp < nsamps; samp++){

		len = mp4ff_read_sample_getsize(mp4, track, samp);
		if(len >= inpalloc){
			inpalloc = len + 1024;
			if((inp = realloc(inp, inpalloc)) == nil)
				sysfatal("No memory, %r");
		}

		if(mp4ff_read_sample_v2(mp4, track, samp, inp) != len)
			sysfatal("cannot extract alac from mp4 container");

		if((dec = NeAACDecDecode(aac, &inf, inp, len)) == nil || inf.error != 0)
			sysfatal("%s cannot decode AAC audio, %s", file,
				NeAACDecGetErrorMessage(inf.error));

		for(i = 0; i < inf.samples; i += inf.channels){
			out[o++] = dec[i];
			out[o++] = dec[i] >> 8;

			if(inf.channels == 1){
				out[o++] = dec[i];
				out[o++] = dec[i] >> 8;
			}
			else{
				out[o++] = dec[i+1];
				out[o++] = dec[i+1] >> 8;
			}

			if(o >= Bufsiz-4){
				if(write(ofd, out, o) != o)
					sysfatal("write failed");
				o = 0;
			}
		}
	}

	if(write(ofd, out, o) != o)
		sysfatal("write failed");

	free(inp);
	NeAACDecClose(aac);
	return 0;
}

static int
pack(uchar *op, uchar *ip, int depth)
{
	switch(depth){
	case 2:
		op[0] = ip[0];
		op[1] = ip[1];
		break;
	case 4:
		op[0] = ip[2];
		op[1] = ip[3];
		break;
	}
	return 2;
}


static void
mp4alac(mp4ff_t mp4, char *file, int track)
{
	enum {
		Stereo = 2,		/* we need stereo */
		Twobytes = 2	/* 16 bits / sample */
	};

	ALACDecoder alac;
	uint got, nmagic;
	uchar *inp, *dec, *magic, out[Bufsiz];
	int nsamps, samp, i, o, len, inpalloc;
	int chans, frmsz, outsz, depth;

	if((alac = alac_create_decoder()) == nil)
		sysfatal("cannot create ALAC decoder");

	if(mp4ff_get_decoder_config(mp4, track, &magic, &nmagic) != 0)
		sysfatal("%s track=%d cannot get config", file, track);

	if(alac_decoder_init(alac, magic, nmagic) != 0)
		sysfatal("cannot initialise ALAC decoder");

	chans = alac_get_channel_count(alac);
	depth = alac_get_bytes_per_sample(alac);
	frmsz = alac_get_frame_size(alac);
	if(frmsz != 0x1000){
		fprint(2, "%s: %s 0x%x bad frame size, coerced to 0x1000\n", argv0, file, frmsz);
		frmsz = 0x1000;
	}

	o = 0;
	inp = nil;
	nsamps = mp4ff_num_samples(mp4, track);
	if((dec = malloc(frmsz * chans * depth)) == nil)
		sysfatal("No memory, %r");

	for(samp = 0; samp < nsamps; samp++){

		len = mp4ff_read_sample_getsize(mp4, track, samp);
		if(len >= inpalloc){
			inpalloc = len + 1024;
			if((inp = realloc(inp, inpalloc)) == nil)
				sysfatal("No memory, %r");
		}

		if(mp4ff_read_sample_v2(mp4, track, samp, inp) != len)
			sysfatal("cannot extract ALAC from MP4 container");

		alac_decode_frame(alac, inp, len, dec, frmsz, Stereo, &got);

		for(i = 0; i < got * Stereo * depth; i += Stereo * depth){
			if(depth == Twobytes){
				out[o++] = dec[i+0];
				out[o++] = dec[i+1];
				out[o++] = dec[i+2];
				out[o++] = dec[i+3];
			}
			else{
				out[o++] = dec[i+2];
				out[o++] = dec[i+3];
				out[o++] = dec[i+6];
				out[o++] = dec[i+7];
			}

			if(o >= Bufsiz-4){
				if(write(ofd, out, o) != o)
					sysfatal("write failed");
				o = 0;
			}
		}
	}
	if(write(ofd, out, o) != o)
		sysfatal("write failed");

	free(inp);
	free(dec);
	alac_destroy_decoder(alac);
}


static void
setrate(mp4ff_t *mp4, int track)
{
	int rate;

	rate = mp4ff_get_sample_rate(mp4, track);
	if(vfd < 0)
		fprint(2, "warning: audio sample rate is %d Hz\n", rate);
	else{
		fprint(vfd, "speed %d\n", rate);
		fprint(cfd, "rate %d\n", rate);
	}
}


static void
play(FILE *fp, char *file)
{
	mp4ff_t *mp4;
	mp4ff_callback_t cb;
	int content, t, type, ntracks;

	memset(&cb, 0, sizeof(cb));
	cb.read = doread;
	cb.seek = doseek;
	cb.user_data = fp;

	if((mp4 = mp4ff_open_read(&cb)) == nil)
		sysfatal("%s cannot open/read, %r", file);

	ntracks = mp4ff_total_tracks(mp4);
	for(t = 0; t < ntracks; t++){
		if(mp4ff_get_track_type(mp4, t) != MP4FF_TRACK_AUDIO)
			continue;
	
		type = mp4ff_get_audio_type(mp4, t);
		switch(type){
		case 0x40:
			setrate(mp4, t);
			mp4aac(mp4, file, t);
			goto done;
		case 0x41:
			setrate(mp4, t);
			mp4alac(mp4, file, t);
			goto done;
		default:
			fprint(2, "%s t=%d type=%x unknown content\n", file, t, type);
			break;
		}
	}
	sysfatal("%s no audio found in file", file);

done:
	mp4ff_close(mp4);
}


static void
usage(void)
{
	fprintf(stderr, "%s [-o outfile] [file...]\n", argv0);
	exit(1);
}

int
main(int argc, char *argv[])
{
	int i;
	FILE *fp;
	char *outfile;
	
	outfile = NULL;
	ARGBEGIN{
	case 'o':
		outfile = EARGF(usage());
		break;
	default:
		usage();
	}ARGEND;

	if(outfile){
		if((ofd = creat(outfile, 0644)) < 0)
			sysfatal("%s cannot create, %r", outfile);
	}else{
		if((ofd = open("/dev/audio", O_WRONLY)) < 0)
			sysfatal("/dev/audio cannot open, %r");
		vfd = open("/dev/volume", O_WRONLY);
		cfd = open("/dev/audioctl", O_WRONLY);
	}

	if(argc == 0){
		play(stdin, "standard input");
	}
	else{
		for(i=0; i<argc; i++){
			if((fp = fopen(argv[i], "r")) == nil)
				sysfatal("%s cannot open, %r", argv[i]);
			play(fp, argv[i]);
			fclose(fp);
		}
	}
	return 0;
}
