#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <lib9.h>
#include <fmt.h>

#include "mp4ffv2.h"

static uint
doread(void *v, void *buf, uint len)
{
	return fread(buf, 1, len, v);
}

static uint
doseek(void *v, uvlong off)
{
    return fseek(v, off, SEEK_SET);
}

void
track(mp4ff_t m, int t)
{
	int n;

	if(mp4ff_get_track_type(m, t) != MP4FF_TRACK_AUDIO)
		return;

	n = mp4ff_get_audio_type(m, t);
	switch(n){
	case 0x40:
		print("track%d.content AAC\n",t );
		break;
	case 0x41:
		print("track%d.content ALAC\n", t);
		break;
	case 0x6B:
		print("track%d.content 'MPEG-1 Audio'\n", t);
		break;
	case 0x69:
		print("track%d.content 'MPEG-2 Audio'\n", t);
		break;
	case 0x67:
		print("track%d.content 'AAC LC'\n", t);
		break;
	case 0xa5:
		print("track%d.content AC3\n", t);
		break;
	case 0xad:
		print("track%d.content Opus\n", t);
		break;
	default:
		print("track%d.content unknown-0x%x\n", t, n);
		break;
	}
	if((n = mp4ff_get_avg_bitrate(m, t)) > 0)
		print("track%d.rate %d\n", t, n);
	if((n = mp4ff_get_channel_count(m, t)) > 0)
		print("track%d.chans %d\n", t, n);
	if((n = mp4ff_get_sample_rate(m, t)) > 0)
		print("track%d.freq %d\n", t, n);
}

static void
info(FILE *fp, char *file)
{
	char *p;
	mp4ff_t m;
	mp4ff_callback_t cb;
	int content, t, ntracks;

	memset(&cb, 0, sizeof(cb));
	cb.read = doread;
	cb.seek = doseek;
	cb.user_data = fp;

	if((m = mp4ff_open_read(&cb)) == nil)
		sysfatal("%s cannot open/read, %r", file);

	printf("# %s\n", file);

	if(mp4ff_meta_get_title(m, &p) && p  && *p)
		print("title %q\n", p);
	if(mp4ff_meta_get_artist(m, &p) && p  && *p)
		print("artist %q\n", p);
	if(mp4ff_meta_get_writer(m, &p) && p  && *p)
		print("writer %q\n", p);
	if(mp4ff_meta_get_album(m, &p) && p  && *p)
		print("album %q\n", p);
	if(mp4ff_meta_get_date(m, &p) && p  && *p)
		print("date %q\n", p);
	if(mp4ff_meta_get_tool(m, &p) && p  && *p)
		print("tool %q\n", p);
	if(mp4ff_meta_get_comment(m, &p) && p  && *p)
		print("comment %q\n", p);
	if(mp4ff_meta_get_genre(m, &p) && p  && *p)
		print("genre %q\n", p);
	if(mp4ff_meta_get_track(m, &p) && p  && *p)
		print("track %q\n", p);
	if(mp4ff_meta_get_disc(m, &p) && p  && *p)
		print("disc %q\n", p);
	if(mp4ff_meta_get_compilation(m, &p) && p  && *p)
		print("compilation %q\n", p);
	if(mp4ff_meta_get_tempo(m, &p) && p  && *p)
		print("title %q\n", p);
	if(mp4ff_meta_get_coverart(m, &p) && p  && *p)
		print("'cover art' %q\n", p);

	ntracks = mp4ff_total_tracks(m);
	for(t = 0; t < ntracks; t++)
		track(m, t);

	mp4ff_close(m);
}


static void
usage(void)
{
	fprintf(stderr, "%s [file...]\n", argv0);
	exit(1);
}

int
main(int argc, char *argv[])
{
	FILE *fp;
	int i, fd;
	char *outfile;
	
	outfile = NULL;
	ARGBEGIN{
	default:
		usage();
	}ARGEND;

	quotefmtinstall();
	if(argc == 0){
		info(stdin, "standard input");
	}
	else{
		for(i=0; i<argc; i++){
			if((fp = fopen(argv[i], "r")) == nil)
				sysfatal("%s cannot open, %r", argv[i]);
			info(fp, argv[i]);
			fclose(fp);
		}
	}
	return 0;
}
