#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "dat.h"
#include "fns.h"


/*
 * #EXTM3U
 * #EXT-X-VERSION:2
 * #EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=339200,CODECS="mp4a.40.2"
 * 'http://as-hls-uk-live.edgesuite.net/pool_6/live/bbc_6music/bbc_6music.isml/bbc_6music-audio%3d320000.m3u8?dvr_window_length=24'
 * 
 * #EXTM3U
 * #EXT-X-VERSION:2
 * ## Created with Unified Streaming Platform(version=1.7.3)
 * #EXT-X-MEDIA-SEQUENCE:222541324
 * #EXT-X-ALLOW-CACHE:NO
 * #EXT-X-TARGETDURATION:7
 * #USP-X-TIMESTAMP-MAP:MPEGTS=4798966176,LOCAL=2015-02-18T13:01:07.200000Z
 * #EXT-X-PROGRAM-DATE-TIME:2015-02-18T13:01:07.200000Z
 * #EXTINF:6, no desc
 * bbc_6music-audio=320000-222541324.ts?dvr_window_length=24
 * #EXTINF:6, no desc
 * bbc_6music-audio=320000-222541325.ts?dvr_window_length=24
 * #EXTINF:6, no desc
 * bbc_6music-audio=320000-222541326.ts?dvr_window_length=24
 * #EXTINF:6, no desc
 * bbc_6music-audio=320000-222541327.ts?dvr_window_length=24
 * #EXTINF:6, no desc
 * bbc_6music-audio=320000-222541328.ts?dvr_window_length=24
*/

typedef struct Aux Aux;
typedef struct Record Record;

struct Aux {
	char *base;			/* base url */
	Biobuf *bp;			/* stream */
};

struct Record {
	char *name;
	int (*func)(Sess *, int, Playlist *, char *);
};

static int
datetime(Sess *, int, Playlist *pl, char *args)
{
	pl->ctime = iso2sec(args);
	return 0;
}

static int
sequence(Sess *, int, Playlist *pl, char *args)
{
	pl->seqno = strtoull(args, nil, 10);
	return 0;
}

static int
endlist(Sess *, int, Playlist *pl, char *args)
{
//	pl->current->endlist = 1;
	return 0;
}

static int
discont(Sess *, int, Playlist *, char *)
{
//	pl->current->reset = 1;
	return 0;
}

static int
stream(Sess *s, int conn, Playlist *pl, char *args)
{
	Aux *ma;
	int rc, i, n, bandw;
	char buf[1024], *a[16], *url, *p;

	bandw = 0;
	n = gettokens(args, a, nelem(a), ",");
	for(i = 0; i < n; i++)
		if(cistrncmp(a[i], "BANDWIDTH=", 10) == 0)
			bandw = strtol(a[i]+10, nil, 10);
//	pl->current->bandw = bandw;

	ma = pl->aux;
	do{
		if(Bgetline(ma->bp, buf, sizeof(buf)) == -1)
			return -1;
		p = skipws(buf);
	}while(*p == 0 || *p == '#');

	url = absurl(ma->base, p);
	rc = play(s, conn, url);
	free(url);
	if(rc == 0)
		return 1;		/* played audio successfully */
	return -1;
}

static int
extinf(Sess *, int, Playlist *pl, char *args)
{
	Aux *a;
	Chunk *ch;
	double secs;
	char buf[1024], *title, *p;

	secs = strtod(args, &args);
	if(*args == ',')
		args++;
	title = skipws(args);

	a = pl->aux;
	do{
		if(Bgetline(a->bp, buf, sizeof(buf)) == -1)
			return -1;
		p = skipws(buf);
	}while(*p == 0 || *p == '#');

	ch = emalloc(sizeof(Chunk));
	ch->title = estrdup(title);
	ch->url = absurl(a->base, buf);
	ch->secs = secs;

	*(pl->tail) = ch;
	pl->tail = &ch->next;

	return 0;
}

Record Records[] = {
	{ "#EXT-X-STREAM-INF:",			stream },
	{ "#EXT-X-MEDIA-SEQUENCE:",		sequence },
	{ "#EXT-X-PROGRAM-DATE-TIME:",	datetime },
	{ "#EXT-X-DISCONTINUITY:",		discont },
	{ "#EXT-X-ENDLIST:",			endlist },
	{ "#EXTINF:",					extinf },
	{ nil,							nil },
};


int
m3u8(Sess *s, Playlist *pl, int conn, char *url, int fd)
{
	Aux *a;
	int n, rc;
	Record *r;
	Biobuf net;
	char buf[1024], *p, *rec;

	pl->tail = &pl->chunks;

	a = emalloc(sizeof(Aux)+ strlen(url)+1);
	pl->aux = a;

	Binit(&net, fd, OREAD);
	a->bp = &net;

	a->base = (char *)&a[1];
	strcpy(a->base, url);

	if((p = strstr(a->base, "://")) == nil)
		sysfatal("%s bad url\n", url);
	p += 3;
	if((p = strrchr(p, '/')) == nil)
		sysfatal("%s bad url\n", url);
	*p = 0;

	rc = 0;
	while(Bgetline(a->bp, buf, sizeof(buf)) != -1){
		rec = skipws(buf);
		if(*rec == 0)
			continue;

		if(*rec == '#'){
			for(r = Records; r->name; r++){
				n = strlen(r->name);
				if(strncmp(rec, r->name, n) == 0){
					n = r->func(s, conn, pl, rec + n);
					switch(n){
					case -1:		// failed
						rc = -1;
						break;
					case 0:			// nothing useful to report
						break;
					case 1:			// played some audio
						if(rc == -1)
							rc = 0;
						rc++; 
						break;
					}
				}
			if(Debug && r->name == nil)
				fprint(2, "%s: %s <ignored>\n", argv0, rec);
			}
		}
	}

	Bterm(&net);
	return rc;
}

