#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "dat.h"
#include "fns.h"

typedef struct Aux Aux;
struct Aux {
	int slot;
};

static char *
getval(char *s)
{
	char *p, *e;

	for(p = s; *p && *p != '='; p++)
		continue;
	if(*p != '='){
		werrstr("%s - no '=' found", s);
		return nil;
	}
	p++;

	while(isspace(*p))
		p++;

	if(*p == '"' || *p == '\''){
		for(e = p; *e && *e != *p; e++)
			continue;
		if(*e != 0){
			werrstr("%s - unmatched quote", s);
			return nil;
		}
		*e = 0;
		p++;
	}
	return p;
}

static Chunk *
lookup(Chunk **root, char *s)
{
	int n;
	Aux *a;
	Chunk *ch, **prev;

	n = atoi(s);
	for(ch = *root; ch; ch = ch->next){
		a = ch->aux;
		if(a->slot == n)
			break;
	}

	if(ch == nil){
		/* NB: insert in correct place in sorted list */
		prev = root;
		for(ch = *root; ch; ch = ch->next){
			a = ch->aux;
			if(a->slot > n)
				break;
			prev = &ch->next;
		}
		ch = emalloc(sizeof(Chunk));
		a = emalloc(sizeof(Aux));
		ch->aux = a;
		a->slot = n;

		ch->next = *prev;
		*prev = ch;
	}

	return ch;	
}


int
pls(Sess *, Playlist *pl, int, char *, int fd)
{
	Chunk *ch;
	Biobuf bin;
	char *p, *line, buf[1024];

	Binit(&bin, fd, OREAD);
	while(Bgetline(&bin, buf, sizeof(buf)) != -1){

		line = skipws(buf);
		if(cistrncmp(line, "Title", 5) == 0){
			ch = lookup(&pl->chunks, line+5);
			p = getval(line+5);
			if(p == nil)
				break;
			ch->title = estrdup(p);
		}
		if(cistrncmp(line, "File", 4) == 0){
			ch = lookup(&pl->chunks, line+4);
			p = getval(line+4);
			if(p == nil)
				break;
			ch->url = estrdup(p);
		}
		if(cistrncmp(line, "Length", 6) == 0){
			ch = lookup(&pl->chunks, line+6);
			p = getval(line+6);
			if(p == nil)
				break;
			ch->secs = strtod(p, nil);
		}
	}
	Bterm(&bin);
	return 0;
}
