#include <u.h>
#include <libc.h>
#include <bio.h>
#include <json.h>
#include "dat.h"
#include "fns.h"

static State Show = { "show",	"" };
static State Title = { "title",	"" };
static State Artist = { "artist",	"" };
static State Disp = { "disp",	"" };
static State Length = { "length",	"" };

void
poll(char *url)
{
	int n;
	Jvalue *j;
	int fd, minsec, maxsec;
	char *s, *a[4], etag[1024], err[ERRMAX];

	srand(getpid() ^ time(nil));

	*etag = 0;
	minsec = 20;
	maxsec = 25;
	for(; 1; sleep(minsec + (((maxsec-minsec) * rand()) / 0x7fff))){
		if((fd = httpredial(url, etag)) < 0){
			rerrstr(err, sizeof(err));
			n = tokenize(err, a, nelem(a));
			if(Debug && n > 2 &&  atoi(a[1]) != 304)		/* 304 == Not modified */
				fprint(2, "%s: httpredial failed, %r\n", argv0);
			continue;
		}

		if((j = json_parse(fd, "")) == nil){
			fprint(2, "%s: json parse failed, %r\n", argv0);
			close(fd);
			continue;
		}
		if(httphdr("etag", etag, sizeof(etag)) < 0)
			*etag = 0;
		close(fd);

		if((s = json_strval(j, "message.duration")) != nil)
			upd(&Length, s);
		if((s = json_strval(j, "message.trackExtended.title")) != nil)
			upd(&Title, s);
		if((s = json_strval(j, "message.trackExtended.artist")) != nil)
			upd(&Artist, s);
		if((s = json_strval(j, "message.programme.brand_title")) != nil)
			upd(&Show, s);
		if((s = json_strval(j, "message.programme.displayTitles")) != nil)
			upd(&Disp, s);
		if((s = json_strval(j, "requestMaxMilliseconds")) != nil)
			maxsec = atoi(s);
		if((s = json_strval(j, "requestMinMilliseconds")) != nil)
			minsec = atoi(s);
		json_free(j);

		Bflush(&Out);
	}
}
