#include <u.h>
#include <libc.h>
#include <bio.h>
#include <json.h>
#include "dat.h"
#include "fns.h"

static State Show = { "show",	"" };
static State Title = { "title",	"" };
static State Artist = { "artist",	"" };

/*
 * BBC's OnAir json feed example.
 * http://polling.bbc.co.uk/modules/onairpanel/include/bbc_radio_one.json
 * 
 * {
 *     "state" : "normal",
 *     "sid" : "bbc_radio_one",
 *     "nowId" : "b03vq0x5",
 *     "nowTitle" : "Fearne Cotton",
 *     "nowLink" : "\/\/www.bbc.co.uk\/showrammes\/b03vq0x5",
 *     "nowInfo" : "A Song About Love by Jake Bugg",
 *     "nextLink" : "\/\/www.bbc.co.uk\/showrammes\/b03vq0x7",
 *     "nextStart" : "2014-03-06T12:45:00+0000",
 *     "nextTitle" : "Newsbeat",
 *     "realtime" : "now_playing"
 * }
 *
 */

int
split(char *str, char **arr, int num, char *delim)
{
	char *p;
	int n, len;

	if(Debug)
		fprint(2, "split: '%s'\n", str);

	if((p = strstr(str, delim)) == nil)
		return 0;

	n = 0;
	len = strlen(delim);
	while(p && n < num){
		*p = 0;
		arr[n++] = str;
		str = p + len;
		p = strstr(str, delim);
	}
	if(n < num)
		arr[n++] = str;

	if(Debug)
		fprint(2, "split: -> n=%d\n", n);
	return n;
}

void
crack(char *title, char *info)
{
	int n;
	char *arr[4];

	if(title){
		if((n = split(title, arr, nelem(arr), " by ")) == 0)
			n = split(title, arr, nelem(arr), " - ");
		if(n == 0)
			upd(&Show, title);
		if(n == 1)
			upd(&Show, arr[0]);
		if(n >= 2){
			upd(&Show, arr[0]);
			upd(&Title, arr[1]);
		}
	}

	if(info){
		if((n = split(info, arr, nelem(arr), " by ")) == 0)
			n = split(info, arr, nelem(arr), " - ");
		if(n == 0){
			upd(&Artist, info);
			upd(&Title, "");
		}
		if(n == 1){
			upd(&Artist, arr[0]);
			upd(&Title, "");
		}
		if(n >= 2){
			upd(&Title, arr[0]);
			upd(&Artist, arr[1]);
	
			/* show is the first part of info - hapepns on R4 plays */
			if(strcmp(Artist.val, Show.val) == 0){
				upd(&Artist, Title.val);
				upd(&Title, "");
			}
		}
	}
}

void
poll(char *url)
{
	int fd;
	Jvalue *j;
	char etag[1024];

	*etag = 0;
	for(; 1; sleep(30*1000)){			/* The BBC's own javascript polls every 30 secs */

		if((fd = httpredial(url, etag)) < 0)
			continue;

		if((j = json_parse(fd, "")) == nil){
			fprint(2, "%s: json parse failed\n", argv0);
			close(fd);
			continue;
		}
		if(httphdr("etag", etag, sizeof(etag)) < 0)
			*etag = 0;
		close(fd);

		crack(json_strval(j, "nowTitle"), json_strval(j, "nowInfo"));
		json_free(j);

		Bflush(&Out);
	}
}
