#include <u.h>
#include <libc.h>
#include <bio.h>
#include "dat.h"
#include "fns.h"

int Conid;
char *Webfs;
Biobuf Out;
int Debug;

void
upd(State *st, char *val)
{
	if(val == nil)
		return;

	if(strcmp(val, st->val) == 0)
		return;
	snprint(st->val, sizeof(st->val), "%s", val);
	Bprint(&Out, "%s %q\n", st->nam, val);
}


int
httpredial(char *url, char *etag)
{
	int e, n, fd;
	char buf[128], *a[8], err[ERRMAX];

	if(Debug)
		fprint(2, "dialing %q\n", url);

	snprint(buf, sizeof buf, "%s/%d/ctl", Webfs, Conid);
	if((fd = open(buf, OWRITE)) < 0){
		werrstr("cannot open another HTTP session - %r");
		return -1;
	}
	fprint(fd, "url %s", url);
	if(etag && *etag)
		fprint(fd, "headers If-None-Match: %s", etag);
	close(fd);

	snprint(buf, sizeof buf, "%s/%d/body", Webfs, Conid);
	fd = open(buf, OREAD);

	if(fd < 0){
		rerrstr(err, sizeof(err));
		if(Debug)
			fprint(2, "connect failed err=%s\n", err);

		if((n = tokenize(err, a, nelem(a))) >= 3){
			e = atoi(a[1]);
			if(e == 0 && strcmp(a[n-1], "interrupted") == 0)
				werrstr("timeout on connect");
		}
		return -1;
	}
	return fd;
}

int
httphdr(char *field, char *val, int len)
{
	int n, fd;
	char buf[128];

	if(val && len > 0)
		*val = 0;

	snprint(buf, sizeof(buf), "%s/%d/%s", Webfs, Conid, field);
	if((fd = open(buf, OREAD)) < 0)
		return -1;
	n = read(fd, val, len-1);
	close(fd);

	if(n < 0)
		return -1;
	val[n] = 0;
	return 0;
}

static void
usage(void)
{
	fprint(2, "usage: %s [-opts] http://...\n", argv0);
	exits("usage");
}

void
main(int argc, char *argv[])
{
	int n, net;
	char buf[128];

	Webfs = "/mnt/web";
	ARGBEGIN{
	case 'd':
		Debug++;
		break;
	case 'w':
		Webfs = EARGF(usage());
		break;
	default:
		usage();
	}ARGEND;

	if(argc != 1)
		usage();

	quotefmtinstall();
	Binit(&Out, 1, OWRITE);

	snprint(buf, sizeof buf, "%s/clone", Webfs);
	if((net = open(buf, ORDWR)) < 0)
		sysfatal("%s cannot open - %r", buf);

	if((n = read(net, buf, sizeof buf-1)) < 0)
		sysfatal("%s cannot read - %r", buf);
	if(n == 0)
		sysfatal("%s is empty - %r", buf);
	buf[n] = '\0';
	Conid = atoi(buf);

	poll(argv[0]);

	close(net);
	exits(nil);
}
