#include <u.h>
#include <libc.h>

/* TODO: should have EBU stereo ident - 250ms silence every 3 seconds on the left channel */

/* the odd '-o file' syntax is for compatibility with the audio decoders */
void
usage(void)
{
	fprint(2, "usage: %s [-o file] [-s hz] [-f hz] [-l db]\n", argv0);
	fprint(2, "   -r rate   set sample rate (default 44100)\n");
	fprint(2, "   -h hz     set tone frequency (default 1000)\n");
	fprint(2, "   -d db     set tone level (default -18dBFs)\n");
	fprint(2, "   -o file   send output to file (default /dev/audio)\n");
	exits("usage");
}

/*
 * write two samples for left and right,
 * big endian as that is what /dev/audio wants.
 * Also write the second halfcycle of the
 * sine wave, offset and inverted.
 */
void
wr2(short **buf1, short **buf2, int n)
{
	char *b1, *b2;

	b1 = (char *)*buf1;
	b1[0] = b1[2] = n & 0xff;
	b1[1] = b1[3] = (n >> 8) & 0xff;
	*buf1 += 2;

	n = -n;

	b2 = (char *)*buf2;
	b2[0] = b2[2] = n & 0xff;
	b2[1] = b2[3] = (n >> 8) & 0xff;
	*buf2 += 2;
}

void
main(int argc, char *argv[])
{
	char *file;
	short *p, *q, *buf;
	int afd, vfd, i, n,  rate, hz;
	double db, gain, phi, twopi, step;

	twopi = 4*atan(1);

	/*
	 * The UK standard is for on line up tone at -18dBfs
	 * whilst the USA uses -20dBfs. I have no data about
	 * the rest of the world (sorry)
	 */
	db = -18;
	hz = 1000;
	rate = 44100;
	file = nil;
	ARGBEGIN{
	case 'r':
		rate = atoi(EARGF(usage()));
		break;
	case 'h':
		hz = atoi(EARGF(usage()));
		break;
	case 'd':
		db = atof(EARGF(usage()));
		break;
	case 'o':
		file = EARGF(usage());
		break;
	default:
		usage();
		break;
	}ARGEND;

	if(argc != 0)
		usage();

	n = rate / hz;
	if((buf = mallocz(n*sizeof(short)*2, 0)) == nil)
		sysfatal("No memory %r\n");

	p = buf;
	q = buf + n;
	phi = 0;
	step = (twopi / n) *2;
	gain = pow(10, db / 20) * 32767;

	for(i = 0; i < n/2; i++){
		wr2(&p, &q, sin(phi)*gain);
		phi += step;
	}

	if(file == nil){
		if((vfd = open("/dev/volume", OWRITE)) != -1){
			fprint(vfd, "speed %d\n", rate);
			close(vfd);
		}
		else
			fprint(2, "%s: warning: cannot set /dev/volume sample rate to %d %r\n", argv0, rate);
		file = "/dev/audio";
	}

	if((afd = create(file, OWRITE, 0644)) == -1)
		sysfatal("%s cannot open %r\n", file);
	while(1)
		write(afd, buf, n*sizeof(short)*2);
}
