/*
 * Play WAV files, supports uncompressed files only
 *
 * May be wirth extracing the cue and list entries so
 * can prent metadata.
 */

#include <u.h>
#include <libc.h>

static int verbose;
static int ofd = -1;
static int vfd = -1;
static int cfd = -1;

ulong
r2(int fd)
{
	uchar buf[2];
	int n;

	n = readn(fd, buf, 2);
	if(n == 0)
		sysfatal("unexpected EOF on input");
	if(n != 2)
		sysfatal("getcc failed, %r");
	return buf[0] | buf[1]<<8;
}

ulong
r4(int fd)
{
	int n;
	uchar buf[4];

	n = readn(fd, buf, 4);
	if(n == 0)
		sysfatal("unexpected EOF on input");
	if(n != 4)
		sysfatal("r4 failed, %r");
	return buf[0] | buf[1]<<8 | buf[2]<<16 | buf[3]<<24;
}

uchar*
getcc(int fd, uchar buf[4])
{
	int n;

	n = readn(fd, buf, 4);
	if(n == 0)
		sysfatal("unexpected EOF on input");
	if(n != 4)
		sysfatal("getcc failed, %r");
	return buf;
}


int
getsamp(uchar *i, int bytes)
{
	int s, n, bits;

	s = 0;
	bits = bytes * 8;
	for(n = 0; n < bits; n += 8)
		s |= *i++ << n;

	s = (s << 8) >> 8;		/* propogate the sign bit */
	return s;
}

int
repack(uchar *out, uchar *inp, int n, int bits, int chans, int framesz)
{
	int s, m, bytes;
	uchar *i, *o, *end;

	o = out;
	end = &inp[n];
	bytes = (bits / 8) + (((bits % 8) == 0)? 0: 1);

	/*
	 * NB: 8bit files are always unsigned, other depths
	 * are always signed (Bill, sit on the naughty step!).
	 */
	switch(bytes){
	case 1:	
		for(i = inp; i < end; i += framesz){
			s = getsamp(i, bytes)-0x80;
			*o++ = 0;
			*o++ = s;
			if(chans > 1)
				s = getsamp(i, bytes)-0x80;
			*o++ = 0;
			*o++ = s;
		}
		break;
	case 2:
		for(i = inp; i < end; i += framesz){
			s = getsamp(i, bytes);
			*o++ = s;
			*o++ = s >> 8;
			if(chans > 1)
				s = getsamp(i, bytes);
			*o++ = s;
			*o++ = s >> 8;
		}
		break;
	case 3:
		for(i = inp; i < end; i += framesz){
			s = getsamp(i, bytes);
			*o++ = s >> 8;
			*o++ = s >> 16;
			if(chans > 1)
				s = getsamp(i, bytes);
			*o++ = s >> 8;
			*o++ = s >> 16;
		}
		break;
	case 4:
		for(i = inp; i < end; i += framesz){
			s = getsamp(i, bytes);
			*o++ = s >> 16;
			*o++ = s >> 24;
			if(chans > 1)
				s = getsamp(i, bytes);
			*o++ = s >> 16;
			*o++ = s >> 24;
		}
		break;
	}
	m = (char *)o - (char *)out;
	return m;
}

void
play(int fd, char *file)
{
	enum { Fmtsz = 2+2+4+4+2+2 };
	int n, m, len, tot, bufsiz;
	uchar *inp, *out, tag[1024];
	int fmt, chans, rate, framesz, bits;

	if(memcmp(getcc(fd, tag), "RIFF", 4) != 0)
		sysfatal("%s: not RIFF file", file);
	tot = r4(fd);
	if(memcmp(getcc(fd, tag), "WAVE", 4) != 0)
		sysfatal("%s: not a WAVE file", file);
	tot -= 4;

	inp = out = nil;
	framesz = chans = bits = 0;

	for(;tot > 0;){
		getcc(fd, tag);
		len = r4(fd);
		tot -= len +8;

		if(memcmp(tag, "data", 4) == 0){
			if(framesz == 0)
				sysfatal("%s data tag with no preceeding fmp tag", file);
			bufsiz = 8192 * framesz;

			if((inp = realloc(inp, bufsiz)) == nil)
				sysfatal("No memory, %r");
			if((out = realloc(out, bufsiz *2 *2)) == nil)
				sysfatal("No memory, %r");

			for( ;len > 0; len -= n){
				n = bufsiz;
				if(n > len)
					n = len;
				if((n = read(fd, inp, n)) <= 0)
					sysfatal("%s: audio read failed, %r", file);
				m = repack(out, inp, n, bits, chans, framesz);
				if(write(ofd, out, m) != m)
					sysfatal("audio write failed, %d %r", m);
			}
		}

		if(memcmp(tag, "fmt ", 4) == 0){
			if(len < Fmtsz)
				sysfatal("%s: format chunk too small", file);
			fmt = r2(fd);
			chans = r2(fd);
			rate = r4(fd);			/* sampling frequency */
			r4(fd);					/* bytes per second (for info only) */
			framesz = r2(fd);
			bits = r2(fd);

			if(verbose){
				print("enc wav\n");
				print("chan %d\n", chans);
				print("freq %f\n", (double)rate / 1000.0);
				print("rate %f\n", (rate * chans * bits) / 1024.0);
			}
			if(vfd < 0)
				fprint(2, "warning: audio sample rate is %d Hz\n", rate);
			else{
				if(fprint(vfd, "speed %d\n", rate) < 0 && 
				   fprint(cfd, "rate %d\n", rate) < 0)
					sysfatal("%d cannot set sample rate, %r", rate);
			}
			if(fmt != 1)
				sysfatal("%s: format=%d not supported", file, fmt);

			len -= Fmtsz;
		}

		/* do not use seek(2) so we can read a stream */
		while(len > 0){
			n = sizeof(tag);
			if(len < n)
				n = len;
			n = read(fd, tag, n);
			if(n <= 0)
				sysfatal("%s: read failed, %r", file);
			len -= n;
		}
	}
}

void
usage(void)
{
	fprint(2, "%s [-s] [-v] [-o outfile] [file...]\n", argv0);
	exits("usage");
}

int
main(int argc, char *argv[])
{
	int i, fd;
	char *outfile;

	outfile = nil;
	ARGBEGIN{
	case 'v':
		verbose++;
		break;
	case 's':
		fprint(2, "%s: -s not implemented\n", argv0);
		break;
	case 'o':
		outfile = EARGF(usage());
		break;
	default:
		usage();
	}ARGEND;

	if(outfile){
		if((ofd = create(outfile, OWRITE, 0644)) < 0)
			sysfatal("%s cannot create - %r", outfile);
	}else{
		if((ofd = open("/dev/audio", OWRITE)) < 0)
			sysfatal("/dev/audio cannot open - %r");
		vfd = open("/dev/volume", OWRITE);
		cfd = open("/dev/audioctl", OWRITE);
	}

	if(argc == 0){
		play(0, "standard input");
	}
	else{
		for(i=0; i<argc; i++){
			if((fd = open(argv[i], OREAD)) == -1)
				sysfatal("%s cannot open - %r", argv[i]);
			play(fd, argv[i]);
			close(fd);
		}
	}
	return 0;
}
