#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>

/* http://www.sonicspot.com/guide/wavefiles.html
 *
 * wav files can contain ID3 tags, we ignore them.
 */

enum {
	Triff = 0x46464952,
	Twave = 0x45564157,
	Tfmt = 0x20746d66,
	Tdata = 0x61746164,
	Tlist = 0x5453494c,
	Tinfo = 0x4f464e49,
};

int Debug;

/*
 * Many of these names are historic but might be 
 * useful if we come across some old strange files
 */
struct {
	int n;
	char *s;
} Fmts[] = {
	{ 0x1,	"Microsoft PCM" },
	{ 0x2,	"Microsoft ADPCM" },
	{ 0x3,	"Microsoft IEEE float" },
	{ 0x4,	"Compaq VSELP" },
	{ 0x5,	"IBM CVSD" },
	{ 0x6,	"Microsoft a-Law" },
	{ 0x7,	"Microsoft u-Law" },
	{ 0x8,	"Microsoft DTS" },
	{ 0x9,	"DRM" },
	{ 0xa,	"WMA 9 Speech" },
	{ 0xb,	"Microsoft Windows Media RT Voice" },
	{ 0x10,	"OKI-ADPCM" },
	{ 0x11,	"Intel IMA/DVI-ADPCM" },
	{ 0x12,	"Videologic Mediaspace ADPCM" },
	{ 0x13,	"Sierra ADPCM" },
	{ 0x14,	"Antex G.723 ADPCM" },
	{ 0x15,	"DSP Solutions DIGISTD" },
	{ 0x16,	"DSP Solutions DIGIFIX" },
	{ 0x17,	"Dialoic OKI ADPCM" },
	{ 0x18,	"Media Vision ADPCM" },
	{ 0x19,	"HP CU" },
	{ 0x1a,	"HP Dynamic Voice" },
	{ 0x20,	"Yamaha ADPCM" },
	{ 0x21,	"SONARC Speech Compression" },
	{ 0x22,	"DSP Group True Speech" },
	{ 0x23,	"Echo Speech Corp." },
	{ 0x24,	"Virtual Music Audiofile AF36" },
	{ 0x25,	"Audio Processing Tech." },
	{ 0x26,	"Virtual Music Audiofile AF10" },
	{ 0x27,	"Aculab Prosody 1612" },
	{ 0x28,	"Merging Tech. LRC" },
	{ 0x30,	"Dolby AC2" },
	{ 0x31,	"Microsoft GSM610" },
	{ 0x32,	"MSN Audio" },
	{ 0x33,	"Antex ADPCME" },
	{ 0x34,	"Control Resources VQLPC" },
	{ 0x35,	"DSP Solutions DIGIREAL" },
	{ 0x36,	"DSP Solutions DIGIADPCM" },
	{ 0x37,	"Control Resources CR10" },
	{ 0x38,	"Natural MicroSystems VBX ADPCM" },
	{ 0x39,	"Crystal Semiconductor IMA ADPCM" },
	{ 0x3a,	"Echo Speech ECHOSC3" },
	{ 0x3b,	"Rockwell ADPCM" },
	{ 0x3c,	"Rockwell DIGITALK" },
	{ 0x3d,	"Xebec Multimedia" },
	{ 0x40,	"Antex G.721 ADPCM" },
	{ 0x41,	"Antex G.728 CELP" },
	{ 0x42,	"Microsoft MSG723" },
	{ 0x43,	"IBM AVC ADPCM" },
	{ 0x45,	"ITU-T G.726" },
	{ 0x50,	"Microsoft MPEG" },
	{ 0x51,	"RT23 or PAC" },
	{ 0x52,	"InSoft RT24" },
	{ 0x53,	"InSoft PAC" },
	{ 0x55,	"MP3" },
	{ 0x59,	"Cirrus" },
	{ 0x60,	"Cirrus Logic" },
	{ 0x61,	"ESS Tech. PCM" },
	{ 0x62,	"Voxware Inc." },
	{ 0x63,	"Canopus ATRAC" },
	{ 0x64,	"APICOM G.726 ADPCM" },
	{ 0x65,	"APICOM G.722 ADPCM" },
	{ 0x66,	"Microsoft DSAT" },
	{ 0x67,	"Micorsoft DSAT DISPLAY" },
	{ 0x69,	"Voxware Byte Aligned" },
	{ 0x70,	"Voxware AC8" },
	{ 0x71,	"Voxware AC10" },
	{ 0x72,	"Voxware AC16" },
	{ 0x73,	"Voxware AC20" },
	{ 0x74,	"Voxware MetaVoice" },
	{ 0x75,	"Voxware MetaSound" },
	{ 0x76,	"Voxware RT29HW" },
	{ 0x77,	"Voxware VR12" },
	{ 0x78,	"Voxware VR18" },
	{ 0x79,	"Voxware TQ40" },
	{ 0x7a,	"Voxware SC3" },
	{ 0x7b,	"Voxware SC3" },
	{ 0x80,	"Soundsoft" },
	{ 0x81,	"Voxware TQ60" },
	{ 0x82,	"Microsoft MSRT24" },
	{ 0x83,	"AT&T G.729A" },
	{ 0x84,	"Motion Pixels MVI MV12" },
	{ 0x85,	"DataFusion G.726" },
	{ 0x86,	"DataFusion GSM610" },
	{ 0x88,	"Iterated Systems Audio" },
	{ 0x89,	"Onlive" },
	{ 0x8a,	"Multitude, Inc. FT SX20" },
	{ 0x8b,	"Infocom ITS A/S G.721 ADPCM" },
	{ 0x8c,	"Convedia G729" },
	{ 0x8d,	"Not specified congruency, Inc." },
	{ 0x91,	"Siemens SBC24" },
	{ 0x92,	"Sonic Foundry Dolby AC3 APDIF" },
	{ 0x93,	"MediaSonic G.723" },
	{ 0x94,	"Aculab Prosody 8kbps" },
	{ 0x97,	"ZyXEL ADPCM" },
	{ 0x98,	"Philips LPCBB" },
	{ 0x99,	"Studer Professional Audio Packed" },
	{ 0xa0,	"Malden PhonyTalk" },
	{ 0xa1,	"Racal Recorder GSM" },
	{ 0xa2,	"Racal Recorder G720.a" },
	{ 0xa3,	"Racal G723.1" },
	{ 0xa4,	"Racal Tetra ACELP" },
	{ 0xb0,	"NEC AAC NEC Corporation" },
	{ 0xff,	"AAC" },
	{ 0x100,	"Rhetorex ADPCM" },
	{ 0x101,	"IBM u-Law" },
	{ 0x102,	"IBM a-Law" },
	{ 0x103,	"IBM ADPCM" },
	{ 0x111,	"Vivo G.723" },
	{ 0x112,	"Vivo Siren" },
	{ 0x120,	"Philips Speech Processing CELP" },
	{ 0x121,	"Philips Speech Processing GRUNDIG" },
	{ 0x123,	"Digital G.723" },
	{ 0x125,	"Sanyo LD ADPCM" },
	{ 0x130,	"Sipro Lab ACEPLNET" },
	{ 0x131,	"Sipro Lab ACELP4800" },
	{ 0x132,	"Sipro Lab ACELP8V3" },
	{ 0x133,	"Sipro Lab G.729" },
	{ 0x134,	"Sipro Lab G.729A" },
	{ 0x135,	"Sipro Lab Kelvin" },
	{ 0x136,	"VoiceAge AMR" },
	{ 0x140,	"Dictaphone G.726 ADPCM" },
	{ 0x150,	"Qualcomm PureVoice" },
	{ 0x151,	"Qualcomm HalfRate" },
	{ 0x155,	"Ring Zero Systems TUBGSM" },
	{ 0x160,	"Microsoft Audio1" },
	{ 0x161,	"Windows Media Audio V2 V7 V8 V9 / DivX audio (WMA) / Alex AC3 Audio" },
	{ 0x162,	"Windows Media Audio Professional V9" },
	{ 0x163,	"Windows Media Audio Lossless V9" },
	{ 0x164,	"WMA Pro over S/PDIF" },
	{ 0x170,	"UNISYS NAP ADPCM" },
	{ 0x171,	"UNISYS NAP ULAW" },
	{ 0x172,	"UNISYS NAP ALAW" },
	{ 0x173,	"UNISYS NAP 16K" },
	{ 0x174,	"MM SYCOM ACM SYC008 SyCom Technologies" },
	{ 0x175,	"MM SYCOM ACM SYC701 G726L SyCom Technologies" },
	{ 0x176,	"MM SYCOM ACM SYC701 CELP54 SyCom Technologies" },
	{ 0x177,	"MM SYCOM ACM SYC701 CELP68 SyCom Technologies" },
	{ 0x178,	"Knowledge Adventure ADPCM" },
	{ 0x180,	"Fraunhofer IIS MPEG2AAC" },
	{ 0x190,	"Digital Theater Systems DTS DS" },
	{ 0x200,	"Creative Labs ADPCM" },
	{ 0x202,	"Creative Labs FASTSPEECH8" },
	{ 0x203,	"Creative Labs FASTSPEECH10" },
	{ 0x210,	"UHER ADPCM" },
	{ 0x215,	"Ulead DV ACM" },
	{ 0x216,	"Ulead DV ACM" },
	{ 0x220,	"Quarterdeck Corp." },
	{ 0x230,	"I-Link VC" },
	{ 0x240,	"Aureal Semiconductor Raw Sport" },
	{ 0x241,	"ESST AC3" },
	{ 0x250,	"Interactive Products HSX" },
	{ 0x251,	"Interactive Products RPELP" },
	{ 0x260,	"Consistent CS2" },
	{ 0x270,	"Sony SCX" },
	{ 0x271,	"Sony SCY" },
	{ 0x272,	"Sony ATRAC3" },
	{ 0x273,	"Sony SPC" },
	{ 0x280,	"TELUM Telum Inc." },
	{ 0x281,	"TELUMIA Telum Inc." },
	{ 0x285,	"Norcom Voice Systems ADPCM" },
	{ 0x300,	"Fujitsu FM TOWNS SND" },
	{ 0x301,	"Fujitsu (not specified)" },
	{ 0x302,	"Fujitsu (not specified)" },
	{ 0x303,	"Fujitsu (not specified)" },
	{ 0x304,	"Fujitsu (not specified)" },
	{ 0x305,	"Fujitsu (not specified)" },
	{ 0x306,	"Fujitsu (not specified)" },
	{ 0x307,	"Fujitsu (not specified)" },
	{ 0x308,	"Fujitsu (not specified)" },
	{ 0x350,	"Micronas Semiconductors, Inc. Development" },
	{ 0x351,	"Micronas Semiconductors, Inc. CELP833" },
	{ 0x400,	"Brooktree Digital" },
	{ 0x401,	"Intel Music Coder (IMC)" },
	{ 0x402,	"Ligos Indeo Audio" },
	{ 0x450,	"QDesign Music" },
	{ 0x500,	"On2 VP7 On2 Technologies" },
	{ 0x501,	"On2 VP6 On2 Technologies" },
	{ 0x680,	"AT&T VME VMPCM" },
	{ 0x681,	"AT&T TCP" },
	{ 0x700,	"YMPEG Alpha (dummy for MPEG-2 compressor)" },
	{ 0x8ae,	"ClearJump LiteWave (lossless)" },
	{ 0x1000,	"Olivetti GSM" },
	{ 0x1001,	"Olivetti ADPCM" },
	{ 0x1002,	"Olivetti CELP" },
	{ 0x1003,	"Olivetti SBC" },
	{ 0x1004,	"Olivetti OPR" },
	{ 0x1100,	"Lernout & Hauspie" },
	{ 0x1101,	"Lernout & Hauspie CELP codec" },
	{ 0x1102,	"Lernout & Hauspie SBC codec" },
	{ 0x1103,	"Lernout & Hauspie SBC codec" },
	{ 0x1104,	"Lernout & Hauspie SBC codec" },
	{ 0x1400,	"Norris Comm. Inc." },
	{ 0x1401,	"ISIAudio" },
	{ 0x1500,	"AT&T Soundspace Music Compression" },
	{ 0x181c,	"VoxWare RT24 speech codec" },
	{ 0x181e,	"Lucent elemedia AX24000P Music codec" },
	{ 0x1971,	"Sonic Foundry LOSSLESS" },
	{ 0x1979,	"Innings Telecom Inc. ADPCM" },
	{ 0x1c07,	"Lucent SX8300P speech codec" },
	{ 0x1c0c,	"Lucent SX5363S G.723 compliant codec" },
	{ 0x1f03,	"CUseeMe DigiTalk (ex-Rocwell)" },
	{ 0x1fc4,	"NCT Soft ALF2CD ACM" },
	{ 0x2000,	"FAST Multimedia DVM" },
	{ 0x2001,	"Dolby DTS (Digital Theater System)" },
	{ 0x2002,	"RealAudio 1 / 2 14.4" },
	{ 0x2003,	"RealAudio 1 / 2 28.8" },
	{ 0x2004,	"RealAudio G2 / 8 Cook (low bitrate)" },
	{ 0x2005,	"RealAudio 3 / 4 / 5 Music (DNET)" },
	{ 0x2006,	"RealAudio 10 AAC (RAAC)" },
	{ 0x2007,	"RealAudio 10 AAC+ (RACP)" },
	{ 0x2500,	"Reserved range to 0x2600 Microsoft" },
	{ 0x3313,	"makeAVIS (ffvfw fake AVI sound from AviSynth scripts)" },
	{ 0x4143,	"Divio MPEG-4 AAC audio" },
	{ 0x4201,	"Nokia adaptive multirate" },
	{ 0x4243,	"Divio G726 Divio, Inc." },
	{ 0x434c,	"LEAD Speech" },
	{ 0x564c,	"LEAD Vorbis" },
	{ 0x5756,	"WavPack Audio" },
	{ 0x674f,	"Ogg Vorbis (mode 1)" },
	{ 0x6750,	"Ogg Vorbis (mode 2)" },
	{ 0x6751,	"Ogg Vorbis (mode 3)" },
	{ 0x676f,	"Ogg Vorbis (mode 1+)" },
	{ 0x6770,	"Ogg Vorbis (mode 2+)" },
	{ 0x6771,	"Ogg Vorbis (mode 3+)" },
	{ 0x7000,	"3COM NBX 3Com Corporation" },
	{ 0x706d,	"FAAD AAC" },
	{ 0x7a21,	"GSM-AMR (CBR, no SID)" },
	{ 0x7a22,	"GSM-AMR (VBR, including SID)" },
	{ 0xa100,	"Comverse Infosys Ltd. G723 1" },
	{ 0xa101,	"Comverse Infosys Ltd. AVQSBC" },
	{ 0xa102,	"Comverse Infosys Ltd. OLDSBC" },
	{ 0xa103,	"Symbol Technologies G729A" },
	{ 0xa104,	"VoiceAge AMR WB VoiceAge Corporation" },
	{ 0xa105,	"Ingenient Technologies Inc. G726" },
	{ 0xa106,	"ISO/MPEG-4 advanced audio Coding" },
	{ 0xa107,	"Encore Software Ltd G726" },
	{ 0xa109,	"Speex ACM Codec xiph.org" },
	{ 0xdfac,	"DebugMode SonicFoundry Vegas FrameServer ACM Codec" },
	{ 0xe708,	"Unknown -" },
	{ 0xf1ac,	"Free Lossless Audio Codec FLAC" },
	{ 0xfffe,	"Extensible" },
	{ 0xffff,	"Development" },
};

struct Tags {
	char *tag;
	char *str;
} Metas[] = {
	{ "GENR",	"genre" },
	{ "IGNR",	"genre" },
	{ "IPRD",	"album" },
	{ "INAM",	"title" },
	{ "ITRK",	"track" },					/* defacto standard it seems */
	{ "itrk",	"track" },					/* defacto standard it seems */
	{ "TITL",	"title" },
	{ "TLEN",	"duration" },
	{ "YEAR",	"year" },


#ifdef NOTDEF
	{ "IPRD",	"Product" },
	{ "ICRD",	"Created" },
	{ "AGES",	"Rated" },
	{ "CMNT",	"Comment" },
	{ "CODE",	"Encoded By" },
	{ "COMM",	"Comments" },
	{ "DIRC",	"Directory" },
	{ "DISP",	"Sound Scheme Title" },
	{ "DTIM",	"Date Time Original" },
	{ "IARL",	"Archival Location" },
	{ "IART",	"Artist" },
	{ "IAS1",	"First Language" },
	{ "IAS2",	"Second Language" },
	{ "IAS3",	"Third Language" },
	{ "IAS4",	"Fourth Language" },
	{ "IAS5",	"Fifth Language" },
	{ "IAS6",	"Sixth Language" },
	{ "IAS7",	"Seventh Language" },
	{ "IAS8",	"Eighth Language" },
	{ "IAS9",	"Ninth Language" },
	{ "IBSU",	"Base URL" },
	{ "ICAS",	"Default Audio Stream" },
	{ "ICDS",	"Costume Designer" },
	{ "ICMS",	"Commissioned" },
	{ "ICMT",	"Comment" },
	{ "ICNM",	"Cinematographer" },
	{ "ICNT",	"Country" },
	{ "ICOP",	"Copyright" },
	{ "ICRP",	"Cropped" },
	{ "IDIM",	"Dimensions" },
	{ "IDPI",	"Dots Per Inch" },
	{ "IDST",	"Distributed By" },
	{ "IEDT",	"Edited By" },
	{ "IENC",	"Encoded By" },
	{ "IENG",	"Engineer" },
	{ "IFRM",	"Part/Total" },				/* from mediainfo */
	{ "IKEY",	"Keywords" },
	{ "ILGT",	"Lightness" },
	{ "ILGU",	"Logo URL" },
	{ "ILIU",	"Logo Icon URL" },
	{ "ILNG",	"Language" },
	{ "IMBI",	"More Info Banner Image" },
	{ "IMBU",	"More Info Banner URL" },
	{ "IMED",	"Medium" },
	{ "IMIT",	"More Info Text" },
	{ "IMIU",	"More Info URL" },
	{ "IMUS",	"Music By" },
	{ "IPDS",	"Production Designer" },
	{ "IPLT"	"Number Of Colors" },
	{ "IPRO",	"Produced By" },
	{ "IPRT",	"Part/Position" },			/* from mediainfo */
	{ "IRIP",	"Ripped By" },
	{ "IRTD",	"Rating" },
	{ "ISBJ",	"Subject" },
	{ "ISFT",	"Software" },
	{ "ISGN",	"Secondary Genre" },
	{ "ISHP",	"Sharpness" },
	{ "ISRC",	"Source" },
	{ "ISRF",	"Source Form" },
	{ "ISTD",	"Production Studio" },
	{ "ISTR",	"Starring" },
	{ "ITCH",	"Technician" },
	{ "IWMU",	"Watermark URL" },
	{ "IWRI",	"Written By" },
	{ "LANG",	"Language" },
	{ "LOCA",	"Location" },
	{ "PRT1",	"Part" },
	{ "PRT2"	"Number Of Parts" },
	{ "RATE",	"Rate" },
	{ "STAR",	"Starring" },
	{ "STAT",	"Statistics Valid?" },
	{ "TAPE",	"Tape Name" },
	{ "TCDO",	"End Timecode" },
	{ "TCOD",	"Start Timecode" },
	{ "TORG",	"Organization" },
	{ "TRCK",	"Track Number" },
	{ "TURL",	"URL" },
	{ "TVER",	"Version" },
	{ "VMAJ",	"Vegas Version Major" },
	{ "VMIN",	"Vegas Version Minor" },

#endif
};

void
dmp(Biobuf *bi, Biobuf *bo, int len)
{
	int i, n, addr, get;
	char buf[16];

	for(addr = 0; addr < len; addr += sizeof(buf)){
		get = sizeof(buf);
		if(len - addr < get)
			get = len - addr;

		n = Bread(bi, buf, get);
		if(n <= 0)
			break;

		Bprint(bo, "%04x	", addr);

		for(i = 0; i < n; i++)
			Bprint(bo, "%02x ", buf[i]);
		for(i = n; i < sizeof(buf); i++)
			Bprint(bo, "   ");
		for(i = 0; i < n; i++)
			if(isprint(buf[i]))
				Bprint(bo, "%c ", buf[i]);
			else
				Bprint(bo, ". ");
		Bprint(bo, "\n");
	}
}
		
char *
fmtname(int n)
{
	int i;

	for(i = 0; i < nelem(Fmts); i++)
		if(n == Fmts[i].n)
			return Fmts[i].s;
	return "unknown";
}

char *
metaname(char *tag)
{
	int i;

	for(i = 0; i < nelem(Metas); i++)
		if(strcmp(tag, Metas[i].tag) == 0)
			return Metas[i].str;
	return nil;
}

int
rd(Biobuf *bp, int bytes)
{
	uint val;
	int i, n, bits;

	val = 0;
	bits = bytes * 8;
	for(i = 0; i < bits; i += 8){
		if((n = Bgetc(bp)) < 0)
			sysfatal("unexpected EOF on input\n");
		val |= n << i;
	}

	return val;
}

void
rdstr(Biobuf *bp, char *buf, int len)
{
	if(Bread(bp, buf, len)< 0)
		sysfatal("unexpected EOF on input\n");
}

int
rdzstr(Biobuf *bp, char buf[256], int len)
{
	int i, c;

	for(i = 0; i < len; i++){
		c = Bgetc(bp);
		if(c < 0)
			sysfatal("unexpected EOF on input\n");
		buf[i] = c;
		if(c == 0)
			break;
	}
	return i;
}

void
info(Biobuf *bo, Biobuf *bi, char *, int *totlen)
{
	int len;
	char *n, nam[6], val[8192];

	memset(nam, 0, sizeof(nam));
	while(*totlen > 0){
		rdstr(bi, nam, 4);
		*totlen -= 4;

		len = rd(bi, 4);
		*totlen -= 4;

		rdzstr(bi, val, len);
		*totlen -= len;

		/*
		 * RIFF reccod length must be even,
		 * eat padding byte if fitted
		 */
		if(len &1){
			rd(bi, 1);
			*totlen -= 1;
		}

		if((n = metaname(nam)) == nil && ! Debug)
			continue;
		if(n == nil)
			n = nam;
		Bprint(bo, "%q=%q\n", n, val);
	}
}

void
list(Biobuf *bo, Biobuf *bi, char *file, int *totlen)
{
	int tag;

	while(*totlen > 0){
		tag = rd(bi, 4);
		*totlen -= 4;

		switch(tag){
		case Tinfo:
			info(bo, bi, file, totlen);
			break;
		default:
			if(Debug)
				fprint(2, "0x%x %c%c%c%c\n", tag,
					tag&0xff,(tag>>8)&0xff, (tag>>16)&0xff,(tag>>24)&0xff);
			break;
		}
	}
}


void
fmt(Biobuf *bo, Biobuf *bi, char *file, int *totlen)
{
	int rate, fmt, depth, freq, chans;

	if(*totlen < 2+2+4+4+2+2)
		sysfatal("%s: fmt record too short", file);

	fmt = rd(bi, 2);			/* compression format */
	chans = rd(bi, 2);			/* channels */
	freq = rd(bi, 4);			/* sample frequency */
	rate = rd(bi, 4);			/* byte rate */
	rd(bi, 2);					/* block alignment */
	depth = rd(bi, 2);			/* bits per sample */

	Bprint(bo, "coding %q\n", fmtname(fmt));
	Bprint(bo, "chans %d\n", chans);
	Bprint(bo, "depth %d bits\n", depth);
	Bprint(bo, "freq %g KHz\n", freq/1000.0);
	Bprint(bo, "rate %d Kbps\n", (rate *8)/1024);
	
	*totlen -= 2+2+4+4+2+2;
}

void
wave(Biobuf *bo, Biobuf *bi, char *file, int *totlen)
{
	int tag, len;

	while(*totlen > 0){
		tag = rd(bi, 4);
		*totlen -= 4;

		len = rd(bi, 4);
		*totlen -= 4;
		*totlen -= len;

		switch(tag){
		case Tfmt:
			fmt(bo, bi, file, &len);
			break;
		case Tlist:
			list(bo, bi, file, &len);
			break;
		default:
			if(Debug)
				fprint(2, "wave: %d 0x%x %c%c%c%c\n", len, tag,
					tag&0xff,(tag>>8)&0xff, (tag>>16)&0xff,(tag>>24)&0xff);
			break;
		}

		Bseek(bi, len, 1);
	}
}

void
riff(Biobuf *bo, Biobuf *bi, char *file)
{
	int tag, totlen;

	if(rd(bi, 4) != Triff)
		sysfatal("%s: not RIFF file", file);
	totlen = rd(bi, 4);

	Bprint(bo, "# %s\n", file);

	while(totlen > 0){
		tag = rd(bi, 4);
		totlen -= 4;

		switch(tag){
		case Twave:
			wave(bo, bi, file, &totlen);
			break;
		default:
			if(Debug)
				fprint(2, "riff: %d 0x%x %c%c%c%c\n", totlen, tag,
					tag&0xff,(tag>>8)&0xff, (tag>>16)&0xff,(tag>>24)&0xff);
			break;
		}

		Bseek(bi, totlen, 1);
	}
}


static void
usage(void)
{
	fprint(2, "%s [file]\n", argv0);
	exits("usage");
}

void
main(int argc, char *argv[])
{
	Biobuf *bi, *bo, out, inp;

	ARGBEGIN{
	case 'd':
		Debug++;
		break;
	default:
		usage();
		break;
	}ARGEND;

	if(argc > 1)
		usage();

	quotefmtinstall();
	Binit(&out, 1, OWRITE);
	bo = &out;

	if(argc == 0){
		Binit(&inp, 0, OREAD);
		bi = &inp;
		riff(bo, bi, "stdin");
	}
	else{
		if((bi = Bopen(argv[0], OREAD)) == nil)
			sysfatal("%s cannot open %r\n", argv[0]);
		riff(bo, bi, argv[0]);
	}

	exits(nil);
}
